@@ -1,7 +1,6 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2017, Anaconda, Inc. All rights reserved.
-#
-# Powered by the Bokeh Development Team.
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
@@ -54,10 +53,18 @@ from .handler import Handler
 # Globals and constants
 #-----------------------------------------------------------------------------
 
+__all__ = (
+    'CodeHandler',
+)
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
 class CodeHandler(Handler):
     ''' Run source code which modifies a Document
 
@@ -135,11 +142,14 @@ class CodeHandler(Handler):
         '''
 
         '''
-        if self.failed:
-            return
 
         module = self._runner.new_module()
 
+        # If no module was returned it means the code runner has some permanent
+        # unfixable problem, e.g. the configured source code has a syntax error
+        if module is None:
+            return
+
         # One reason modules are stored is to prevent the module
         # from being gc'd before the document is. A symptom of a
         # gc'd module is that its globals become None. Additionally
@@ -198,10 +208,6 @@ class CodeHandler(Handler):
         for f in old:
             setattr(io, f, old[f])
 
-#-----------------------------------------------------------------------------
-# Dev API
-#-----------------------------------------------------------------------------
-
 #-----------------------------------------------------------------------------
 # Private API
 #-----------------------------------------------------------------------------
