@@ -1,7 +1,6 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2017, Anaconda, Inc. All rights reserved.
-#
-# Powered by the Bokeh Development Team.
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
@@ -34,8 +33,6 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import logging
 log = logging.getLogger(__name__)
 
-from bokeh.util.api import general, dev ; general, dev
-
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
@@ -61,12 +58,18 @@ DEFAULT_SESSION_ID = "default"
 
 DEFAULT_SERVER_WEBSOCKET_URL = websocket_url_for_server_url(DEFAULT_SERVER_HTTP_URL)
 
+__all__ = (
+    'ClientSession',
+    'pull_session',
+    'push_session',
+    'show_session',
+)
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-@general((1,0,0))
-def pull_session(session_id=None, url='default', io_loop=None):
+def pull_session(session_id=None, url='default', io_loop=None, arguments=None):
     ''' Create a session by loading the current server-side document.
 
     ``session.document`` will be a fresh document loaded from
@@ -104,19 +107,27 @@ def pull_session(session_id=None, url='default', io_loop=None):
                 can also be `"default"` which will connect to the default app URL
 
         io_loop (``tornado.ioloop.IOLoop``, optional) :
-            The IOLoop to use for the websocket
+            The ``IOLoop`` to use for the websocket
+
+        arguments (dict[str, str], optional) :
+            A dictionary of key/values to be passed as HTTP request arguments
+            to Bokeh application code (default: None)
+
+            Note that should only be provided when pulling new sessions.
+            If ``session_id`` is not None, or a session with ``session_id``
+            already exists, these arguments will have no effect.
 
     Returns:
         ClientSession :
-            A new ClientSession connected to the server
+            A new ``ClientSession`` connected to the server
 
     '''
+
     coords = _SessionCoordinates(session_id=session_id, url=url)
-    session = ClientSession(session_id=session_id, websocket_url=websocket_url_for_server_url(coords.url), io_loop=io_loop)
+    session = ClientSession(session_id=session_id, websocket_url=websocket_url_for_server_url(coords.url), io_loop=io_loop, arguments=arguments)
     session.pull()
     return session
 
-@general((1,0,0))
 def push_session(document, session_id=None, url='default', io_loop=None):
     ''' Create a session by pushing the given document to the server,
     overwriting any existing server-side document.
@@ -161,7 +172,6 @@ def push_session(document, session_id=None, url='default', io_loop=None):
     session.push(document)
     return session
 
-@general((1,0,0))
 def show_session(session_id=None, url='default', session=None, browser=None, new="tab", controller=None):
         ''' Open a browser displaying a session document.
 
@@ -207,18 +217,30 @@ def show_session(session_id=None, url='default', session=None, browser=None, new
         controller.open(server_url + "?bokeh-session-id=" + quote_plus(session_id),
                         new=NEW_PARAM[new])
 
-@general((1,0,0))
 class ClientSession(object):
     ''' Represents a websocket connection to a server-side session.
 
-    Each server session stores a Document, which is kept in sync
-    with the document in this ClientSession instance.
-    Always call either pull() or push() immediately after
-    creating the session, if you construct a session by hand.
+    Each server session stores a Document, which is kept in sync with the
+    corresponding Document for this ``ClientSession`` instance. Updates on
+    either side of the connection will automatically propagate to the other
+    side, as long as the connection is open.
+
+    ClientSession objects can (and usually should) be used as a context manager
+    so that the session is properly closed:
+
+    .. code-block:: python
+
+        with pull_session(url=app_url) as session:
+            # customize session here
+            script = server_session(session_id=mysession.id, url=app_url)
+            return render_template("embed.html", script=script, template="Flask")
+
+    If you do not use ``ClientSession`` in this way, it is up to you to ensure
+    that ``session.close()`` is called.
 
     '''
 
-    def __init__(self, session_id=None, websocket_url=DEFAULT_SERVER_WEBSOCKET_URL, io_loop=None):
+    def __init__(self, session_id=None, websocket_url=DEFAULT_SERVER_WEBSOCKET_URL, io_loop=None, arguments=None):
         ''' A connection which attaches to a particular named session on the
         server.
 
@@ -239,26 +261,45 @@ class ClientSession(object):
 
             io_loop (IOLoop, optional) :
                 The IOLoop to use for the websocket
+
+            arguments (dict[str, str], optional) :
+                A dictionary of key/values to be passed as HTTP request
+                arguments to Bokeh application code (default: None)
+
+                Note that should only be provided when pulling new sessions.
+                If ``session_id`` is not None, or a session with ``session_id``
+                already exists, these arguments will have no effect.
+
         '''
         self._document = None
         self._id = self._ensure_session_id(session_id)
 
         from .connection import ClientConnection
-        self._connection = ClientConnection(session=self, io_loop=io_loop, websocket_url=websocket_url)
+        self._connection = ClientConnection(session=self, io_loop=io_loop, websocket_url=websocket_url, arguments=arguments)
 
         from ..server.callbacks import _DocumentCallbackGroup
         self._callbacks = _DocumentCallbackGroup(self._connection.io_loop)
 
+    def __enter__(self):
+        '''
+
+        '''
+        return self
+
+    def __exit__(self, exc_type, exc_value, exc_traceback):
+        '''
+
+        '''
+        self.close()
+
     # Properties --------------------------------------------------------------
 
     @property
-    @general((1,0,0))
     def connected(self):
         ''' Whether this session is currently connected. '''
         return self._connection.connected
 
     @property
-    @general((1,0,0))
     def document(self):
         ''' A :class:`~bokeh.document.Document` that will be kept in sync with
         the corresponding Document on the server.
@@ -270,24 +311,20 @@ class ClientSession(object):
         return self._document
 
     @property
-    @general((1,0,0))
     def id(self):
         ''' A unique ID for this session. '''
         return self._id
 
     # Public methods ----------------------------------------------------------
 
-    @general((1,0,0))
     def connect(self):
         ''' Connect to a Bokeh server at the configured URL. '''
         self._connection.connect()
 
-    @general((1,0,0))
     def close(self, why="closed"):
         ''' Close the connection to the server. '''
         self._connection.close(why)
 
-    @general((1,0,0))
     def force_roundtrip(self):
         ''' Force a round-trip request/reply to the server, sometimes needed to
         avoid race conditions. Mostly useful for testing.
@@ -301,9 +338,8 @@ class ClientSession(object):
         '''
         self._connection.force_roundtrip()
 
-    @general((1,0,0))
     def loop_until_closed(self, suppress_warning=False):
-        ''' Execute a blocking loop that runs and exectutes event callbacks
+        ''' Execute a blocking loop that runs and executes event callbacks
         until the connection is closed (e.g. by hitting Ctrl-C).
 
         While this method can be used to run Bokeh application code "outside"
@@ -316,7 +352,6 @@ class ClientSession(object):
             warnings.warn(_BOKEH_CLIENT_APP_WARNING_FULL)
         self._connection.loop_until_closed()
 
-    @general((1,0,0))
     def pull(self):
         ''' Pull the server's state and set it as session.document.
 
@@ -338,7 +373,6 @@ class ClientSession(object):
         if self.document is None:
             self._attach_document(doc)
 
-    @general((1,0,0))
     def push(self, document=None):
         ''' Push the given document to the server and record it as session.document.
 
@@ -372,7 +406,6 @@ class ClientSession(object):
         if self._document is None:
             self._attach_document(doc)
 
-    @general((1,0,0))
     def request_server_info(self):
         ''' Ask for information about the server.
 
@@ -382,7 +415,6 @@ class ClientSession(object):
         '''
         return self._connection.request_server_info()
 
-    @general((1,0,0))
     def show(self, obj=None, browser=None, new="tab"):
         ''' Open a browser displaying this session.
 
@@ -409,8 +441,6 @@ class ClientSession(object):
 
     # Internal methods --------------------------------------------------------
 
-    # Private methods ---------------------------------------------------------
-
     def _attach_document(self, document):
         self._document = document
         self._document.on_change_dispatch_to(self)
@@ -464,7 +494,7 @@ The use of `session.loop_until_closed` and `push_session` to run Bokeh
 application code outside a Bokeh server is **HIGHLY DISCOURAGED** for any real
 use.
 
-Running application code outside a Bokeh server with bokeh.client in this way
+Running application code outside a Bokeh server with ``bokeh.client`` in this way
 has (and always will have) several intrinsic drawbacks:
 
 * Fast binary array transport is NOT available! Base64 fallback is much slower
@@ -472,7 +502,7 @@ has (and always will have) several intrinsic drawbacks:
 * Server *and* client process must be running at ALL TIMES for callbacks to work
 * App code run outside the Bokeh server is NOT SCALABLE behind a load balancer
 
-The bokeh.client API is recommended to use ONLY for testing, or for customizing
+The ``bokeh.client`` API is recommended to use ONLY for testing, or for customizing
 individual sessions running in a full Bokeh server, before passing on to viewers.
 
 For information about different ways of running apps in a Bokeh server, see:
