@@ -1,3 +1,9 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Provide a base class for objects that can have declarative, typed,
 serializable properties.
 
@@ -8,42 +14,53 @@ serializable properties.
     anyone who is not directly developing on Bokeh's own infrastructure.
 
 '''
-from __future__ import absolute_import
+
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import logging
 log = logging.getLogger(__name__)
 
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
 import difflib
 import inspect
-from operator import itemgetter
-import sys
 from warnings import warn
 
-from six import StringIO
+# External imports
 
-from ..util.dependencies import import_optional
+# Bokeh imports
 from ..util.future import with_metaclass
 from ..util.string import nice_join
-from .property.containers import PropertyValueContainer
+
 from .property.descriptor_factory import PropertyDescriptorFactory
 from .property.override import Override
+from .property.wrappers import PropertyValueContainer
 
-_ABSTRACT_ADMONITION = '''
-    .. note::
-        This is an abstract base class used to help organize the hierarchy of Bokeh
-        model types. **It is not useful to instantiate on its own.**
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
 
-'''
+__all__ = (
+    'abstract',
+    'accumulate_dict_from_superclasses',
+    'accumulate_from_superclasses',
+    'HasProps',
+    'MetaHasProps',
+)
 
-_EXAMPLE_TEMPLATE = '''
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
 
-    Example
-    -------
-
-    .. bokeh-plot:: ../%(path)s
-        :source-position: below
-
-'''
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
 
 def abstract(cls):
     ''' A decorator to mark abstract base classes derived from |HasProps|.
@@ -329,7 +346,7 @@ class HasProps(with_metaclass(MetaHasProps, object)):
             descriptor = self.lookup(name)
             descriptor.set_from_json(self, json, models, setter)
         else:
-            log.warn("JSON had attr %r on obj %r, which is a client-only or invalid attribute that shouldn't have been sent", name, self)
+            log.warning("JSON had attr %r on obj %r, which is a client-only or invalid attribute that shouldn't have been sent", name, self)
 
     def update(self, **kwargs):
         ''' Updates the object's properties from the given keyword arguments.
@@ -457,7 +474,7 @@ class HasProps(with_metaclass(MetaHasProps, object)):
         properties defined on any parent classes.
 
         Returns:
-            set[str] : names of DataSpec properties
+            set[str] : names of ``DataSpec`` properties
 
         '''
         return set(cls.dataspecs_with_props().keys())
@@ -620,95 +637,6 @@ class HasProps(with_metaclass(MetaHasProps, object)):
         '''
         self.apply_theme(property_values=dict())
 
-    def pretty(self, verbose=False, max_width=79, newline='\n'):
-        ''' Generate a "pretty" string representation of the object.
-
-        .. note::
-            This function only functions in the IPython shell or
-            Jupyter Notebooks.
-
-        Args:
-            Verbose (bool, optional) :
-                This is a conventional argument for IPython representation
-                printers but is unused by Bokeh. (default: False)
-
-            max_width (int, optional) :
-                Minimum width to start breaking lines when possible. (default: 79)
-
-            newline (str, optional) :
-                Character to use to separate each line (default: ``\\n``)
-
-        Returns:
-            str : pretty object representation
-
-        Raises:
-            ValueError, if ``IPython`` cannot be imported
-
-        '''
-        IPython = import_optional('IPython')
-        cls = self.__class__
-
-        if IPython:
-            from IPython.lib.pretty import RepresentationPrinter
-
-            stream = StringIO()
-            printer = RepresentationPrinter(stream, verbose, max_width, newline)
-            printer.pretty(self)
-            printer.flush()
-            return stream.getvalue()
-        else:
-            raise RuntimeError("%s.%s.pretty() requires IPython" % (cls.__module__, cls.__name__))
-
-    def pprint(self, verbose=False, max_width=79, newline='\n'):
-        ''' Print a "pretty" string representation of the object to stdout.
-
-        .. note::
-            This function only functions in the IPython shell or
-            Jupyter Notebooks.
-
-        Args:
-            Verbose (bool, optional) :
-                This is a conventional argument for IPython representation
-                printers but is unused by Bokeh. (default: False)
-
-            max_width (int, optional) :
-                Minimum width to start breaking lines when possible. (default: 79)
-
-            newline (str, optional) :
-                Character to use to separate each line (default: ``\\n``)
-
-        Returns:
-            None
-
-        Raises:
-            ValueError, if ``IPython`` cannot be imported
-
-        Examples:
-
-            .. code-block:: python
-
-                In [1]: from bokeh.models import Range1d
-
-                In [1]: r = Range1d(start=10, end=20)
-
-                In [2]: r.pprint()
-                bokeh.models.ranges.Range1d(
-                    id='1576d21a-0c74-4214-8d8f-ad415e1e4ed4',
-                    bounds=None,
-                    callback=None,
-                    end=20,
-                    js_property_callbacks={},
-                    max_interval=None,
-                    min_interval=None,
-                    name=None,
-                    start=10,
-                    tags=[])
-
-        '''
-        sys.stdout.write(self.pretty())
-        sys.stdout.write(newline)
-        sys.stdout.flush()
-
     def _clone(self):
         ''' Duplicate a HasProps object.
 
@@ -717,25 +645,27 @@ class HasProps(with_metaclass(MetaHasProps, object)):
         '''
         return self.__class__(**self._property_values)
 
-    def _repr_pretty_(self, p, cycle):
-        '''
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
 
-        '''
-        name = "%s.%s" % (self.__class__.__module__, self.__class__.__name__)
+_ABSTRACT_ADMONITION = '''
+    .. note::
+        This is an abstract base class used to help organize the hierarchy of Bokeh
+        model types. **It is not useful to instantiate on its own.**
 
-        if cycle:
-            p.text("%s(...)" % name)
-        else:
-            with p.group(4, '%s(' % name, ')'):
-                props = self.properties_with_values().items()
-                sorted_props = sorted(props, key=itemgetter(0))
-                all_props = sorted_props
-                for i, (prop, value) in enumerate(all_props):
-                    if i == 0:
-                        p.breakable('')
-                    else:
-                        p.text(',')
-                        p.breakable()
-                    p.text(prop)
-                    p.text('=')
-                    p.pretty(value)
+'''
+
+_EXAMPLE_TEMPLATE = '''
+
+    Example
+    -------
+
+    .. bokeh-plot:: ../%(path)s
+        :source-position: below
+
+'''
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
