@@ -1,3 +1,9 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Provide events that represent various changes to Bokeh Documents.
 
 These events are used internally to signal changes to Documents. For
@@ -5,7 +11,54 @@ information about user-facing (e.g. UI or tool) events, see the reference
 for :ref:`bokeh.events`.
 
 '''
-from __future__ import absolute_import
+
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
+
+import logging
+log = logging.getLogger(__name__)
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+
+# External imports
+
+# Bokeh imports
+from ..util.dependencies import import_optional
+
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+pd = import_optional('pandas')
+
+__all__ = (
+    'ColumnDataChangedEvent',
+    'ColumnsStreamedEvent',
+    'ColumnsPatchedEvent',
+    'DocumentChangedEvent',
+    'DocumentPatchedEvent',
+    'ModelChangedEvent',
+    'RootAddedEvent',
+    'RootRemovedEvent',
+    'SessionCallbackAdded',
+    'SessionCallbackRemoved',
+    'TitleChangedEvent',
+    'TitleChangedEvent',
+)
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
 
 class DocumentChangedEvent(object):
     ''' Base class for all internal events representing a change to a
@@ -13,7 +66,7 @@ class DocumentChangedEvent(object):
 
     '''
 
-    def __init__(self, document, setter=None):
+    def __init__(self, document, setter=None, callback_invoker=None):
         '''
 
         Args:
@@ -31,9 +84,21 @@ class DocumentChangedEvent(object):
                 The session can compare the event setter to itself, and
                 suppress any updates that originate from itself.
 
+            callback_invoker (callable, optional) :
+                A callable that will invoke any Model callbacks that should
+                be executed in response to the change that triggered this
+                event. (default: None)
+
         '''
         self.document = document
         self.setter = setter
+        self.callback_invoker = callback_invoker
+
+    def combine(self, event):
+        '''
+
+        '''
+        return False
 
     def dispatch(self, receiver):
         ''' Dispatch handling of this event to a receiver.
@@ -94,7 +159,7 @@ class ModelChangedEvent(DocumentPatchedEvent):
 
     '''
 
-    def __init__(self, document, model, attr, old, new, serializable_new, hint=None, setter=None):
+    def __init__(self, document, model, attr, old, new, serializable_new, hint=None, setter=None, callback_invoker=None):
         '''
 
         Args:
@@ -130,10 +195,16 @@ class ModelChangedEvent(DocumentPatchedEvent):
                 See :class:`~bokeh.document.events.DocumentChangedEvent`
                 for more details.
 
+            callback_invoker (callable, optional) :
+                A callable that will invoke any Model callbacks that should
+                be executed in response to the change that triggered this
+                event. (default: None)
+
+
         '''
         if setter is None and isinstance(hint, (ColumnsStreamedEvent, ColumnsPatchedEvent)):
             setter = hint.setter
-        super(ModelChangedEvent, self).__init__(document, setter)
+        super(ModelChangedEvent, self).__init__(document, setter, callback_invoker)
         self.model = model
         self.attr = attr
         self.old = old
@@ -141,6 +212,28 @@ class ModelChangedEvent(DocumentPatchedEvent):
         self.serializable_new = serializable_new
         self.hint = hint
 
+    def combine(self, event):
+        '''
+
+        '''
+        if not isinstance(event, ModelChangedEvent): return False
+
+        # If these are not true something weird is going on, maybe updates from
+        # Python bokeh.client, don't try to combine
+        if self.setter != event.setter: return False
+        if self.document != event.document: return False
+
+        if self.hint:
+            return self.hint.combine(event.hint)
+
+        if (self.model == event.model) and (self.attr == event.attr):
+            self.new = event.new
+            self.serializable_new = event.serializable_new
+            self.callback_invoker = event.callback_invoker
+            return True
+
+        return False
+
     def dispatch(self, receiver):
         ''' Dispatch handling of this event to a receiver.
 
@@ -210,7 +303,7 @@ class ColumnDataChangedEvent(DocumentPatchedEvent):
 
     '''
 
-    def __init__(self, document, column_source, cols=None, setter=None):
+    def __init__(self, document, column_source, cols=None, setter=None, callback_invoker=None):
         '''
 
         Args:
@@ -230,8 +323,14 @@ class ColumnDataChangedEvent(DocumentPatchedEvent):
                 See :class:`~bokeh.document.events.DocumentChangedEvent`
                 for more details.
 
+            callback_invoker (callable, optional) :
+                A callable that will invoke any Model callbacks that should
+                be executed in response to the change that triggered this
+                event. (default: None)
+
+
         '''
-        super(ColumnDataChangedEvent, self).__init__(document, setter)
+        super(ColumnDataChangedEvent, self).__init__(document, setter, callback_invoker)
         self.column_source = column_source
         self.cols = cols
 
@@ -242,7 +341,7 @@ class ColumnDataChangedEvent(DocumentPatchedEvent):
 
         '''
         super(ColumnDataChangedEvent, self).dispatch(receiver)
-        if hasattr(receiver, '_column_data_changed)'):
+        if hasattr(receiver, '_column_data_changed'):
             receiver._column_data_changed(self)
 
     def generate(self, references, buffers):
@@ -290,7 +389,7 @@ class ColumnsStreamedEvent(DocumentPatchedEvent):
 
     '''
 
-    def __init__(self, document, column_source, data, rollover, setter=None):
+    def __init__(self, document, column_source, data, rollover, setter=None, callback_invoker=None):
         '''
 
         Args:
@@ -300,7 +399,10 @@ class ColumnsStreamedEvent(DocumentPatchedEvent):
             column_source (ColumnDataSource) :
                 The data source to stream new data to.
 
-            data (dict) :
+            data (dict or DataFrame) :
+                New data to stream.
+
+                If a DataFrame, will be stored as ``{c: df[c] for c in df.columns}``
 
             rollover (int) :
                 A rollover limit. If the data source columns exceed this
@@ -314,9 +416,18 @@ class ColumnsStreamedEvent(DocumentPatchedEvent):
                 See :class:`~bokeh.document.events.DocumentChangedEvent`
                 for more details.
 
+            callback_invoker (callable, optional) :
+                A callable that will invoke any Model callbacks that should
+                be executed in response to the change that triggered this
+                event. (default: None)
+
         '''
-        super(ColumnsStreamedEvent, self).__init__(document, setter)
+        super(ColumnsStreamedEvent, self).__init__(document, setter, callback_invoker)
         self.column_source = column_source
+
+        if pd and isinstance(data, pd.DataFrame):
+            data = {c: data[c] for c in data.columns}
+
         self.data = data
         self.rollover = rollover
 
@@ -370,7 +481,7 @@ class ColumnsPatchedEvent(DocumentPatchedEvent):
 
     '''
 
-    def __init__(self, document, column_source, patches, setter=None):
+    def __init__(self, document, column_source, patches, setter=None, callback_invoker=None):
         '''
 
         Args:
@@ -389,8 +500,13 @@ class ColumnsPatchedEvent(DocumentPatchedEvent):
                 See :class:`~bokeh.document.events.DocumentChangedEvent`
                 for more details.
 
+            callback_invoker (callable, optional) :
+                A callable that will invoke any Model callbacks that should
+                be executed in response to the change that triggered this
+                event. (default: None)
+
         '''
-        super(ColumnsPatchedEvent, self).__init__(document, setter)
+        super(ColumnsPatchedEvent, self).__init__(document, setter, callback_invoker)
         self.column_source = column_source
         self.patches = patches
 
@@ -442,7 +558,7 @@ class TitleChangedEvent(DocumentPatchedEvent):
 
     '''
 
-    def __init__(self, document, title, setter=None):
+    def __init__(self, document, title, setter=None, callback_invoker=None):
         '''
 
         Args:
@@ -459,10 +575,31 @@ class TitleChangedEvent(DocumentPatchedEvent):
                 See :class:`~bokeh.document.events.DocumentChangedEvent`
                 for more details.
 
+            callback_invoker (callable, optional) :
+                A callable that will invoke any Model callbacks that should
+                be executed in response to the change that triggered this
+                event. (default: None)
+
+
         '''
-        super(TitleChangedEvent, self).__init__(document, setter)
+        super(TitleChangedEvent, self).__init__(document, setter, callback_invoker)
         self.title = title
 
+    def combine(self, event):
+        '''
+
+        '''
+        if not isinstance(event, TitleChangedEvent): return False
+
+        # If these are not true something weird is going on, maybe updates from
+        # Python bokeh.client, don't try to combine
+        if self.setter != event.setter: return False
+        if self.document != event.document: return False
+
+        self.title = event.title
+        self.callback_invoker = event.callback_invoker
+        return True
+
     def generate(self, references, buffers):
         ''' Create a JSON representation of this event suitable for sending
         to clients.
@@ -499,7 +636,7 @@ class RootAddedEvent(DocumentPatchedEvent):
 
     '''
 
-    def __init__(self, document, model, setter=None):
+    def __init__(self, document, model, setter=None, callback_invoker=None):
         '''
 
         Args:
@@ -516,8 +653,13 @@ class RootAddedEvent(DocumentPatchedEvent):
                 See :class:`~bokeh.document.events.DocumentChangedEvent`
                 for more details.
 
+            callback_invoker (callable, optional) :
+                A callable that will invoke any Model callbacks that should
+                be executed in response to the change that triggered this
+                event. (default: None)
+
         '''
-        super(RootAddedEvent, self).__init__(document, setter)
+        super(RootAddedEvent, self).__init__(document, setter, callback_invoker)
         self.model = model
 
     def generate(self, references, buffers):
@@ -557,7 +699,7 @@ class RootRemovedEvent(DocumentPatchedEvent):
 
     '''
 
-    def __init__(self, document, model, setter=None):
+    def __init__(self, document, model, setter=None, callback_invoker=None):
         '''
 
         Args:
@@ -574,8 +716,14 @@ class RootRemovedEvent(DocumentPatchedEvent):
                 See :class:`~bokeh.document.events.DocumentChangedEvent`
                 for more details.
 
+            callback_invoker (callable, optional) :
+                A callable that will invoke any Model callbacks that should
+                be executed in response to the change that triggered this
+                event. (default: None)
+
+
         '''
-        super(RootRemovedEvent, self).__init__(document, setter)
+        super(RootRemovedEvent, self).__init__(document, setter, callback_invoker)
         self.model = model
 
     def generate(self, references, buffers):
@@ -670,3 +818,11 @@ class SessionCallbackRemoved(DocumentChangedEvent):
         super(SessionCallbackRemoved, self).dispatch(receiver)
         if hasattr(receiver, '_session_callback_removed'):
             receiver._session_callback_removed(self)
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
