@@ -1,15 +1,30 @@
-from __future__ import absolute_import, print_function
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 
-import pytest
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
 
-from mock import patch
+import pytest ; pytest
 
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
 import logging
-import unittest
 
 from copy import copy
+from mock import patch
 
-import bokeh.document.document as document
+# External imports
+
+# Bokeh imports
 
 from bokeh.io.doc import curdoc
 from bokeh.models import ColumnDataSource
@@ -17,12 +32,19 @@ from bokeh.document.events import (ColumnsPatchedEvent, ColumnsStreamedEvent, Mo
                                    RootRemovedEvent, SessionCallbackAdded, SessionCallbackRemoved, TitleChangedEvent)
 from bokeh.protocol.messages.patch_doc import process_document_events
 from bokeh.util.logconfig import basicConfig
-from bokeh.util.warnings import BokehDeprecationWarning
 
 from .setup import AnotherModelInTestDocument, SomeModelInTestDocument, ModelThatOverridesName, ModelWithSpecInTestDocument
 
-# needed for caplog tests to function
-basicConfig()
+# Module under test
+import bokeh.document.document as document
+
+#-----------------------------------------------------------------------------
+# Setup
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
 
 class TestDocumentHold(object):
 
@@ -119,7 +141,9 @@ class Test_Document_delete_modules(object):
         # add an extra referrer for delete_modules to complain about
         extra.append(mod)
         import gc
-        assert len(gc.get_referrers(mod)) == 4
+
+        # get_referrers behavior changed in Python 3.7, see https://github.com/bokeh/bokeh/issues/8221
+        assert len(gc.get_referrers(mod)) in (3,4)
 
         with caplog.at_level(logging.ERROR):
             d.delete_modules()
@@ -129,7 +153,7 @@ class Test_Document_delete_modules(object):
         assert 'junkjunkjunk' not in sys.modules
         assert d._modules is None
 
-class TestDocument(unittest.TestCase):
+class TestDocument(object):
 
     def test_empty(self):
         d = document.Document()
@@ -197,8 +221,8 @@ class TestDocument(unittest.TestCase):
         d.add_root(m)
         assert len(d.roots) == 1
         assert len(d._all_models) == 2
-        assert d.get_model_by_id(m._id) == m
-        assert d.get_model_by_id(m2._id) == m2
+        assert d.get_model_by_id(m.id) == m
+        assert d.get_model_by_id(m2.id) == m2
         assert d.get_model_by_id("not a valid ID") is None
 
     def test_get_model_by_name(self):
@@ -562,33 +586,6 @@ class TestDocument(unittest.TestCase):
         assert isinstance(events[0], SessionCallbackAdded)
         assert isinstance(events[1], SessionCallbackRemoved)
 
-    def test_deprecated_add_remove_periodic_callback(self):
-        d = document.Document()
-
-        events = []
-
-        def listener(event):
-            events.append(event)
-
-        d.on_change(listener)
-
-        assert len(d.session_callbacks) == 0
-        assert not events
-
-        def cb(): pass
-
-        d.add_periodic_callback(cb, 1)
-        assert len(d.session_callbacks) == len(events) == 1
-        assert isinstance(events[0], SessionCallbackAdded)
-        assert d.session_callbacks[0] == events[0].callback
-
-        with pytest.warns(BokehDeprecationWarning):
-            d.remove_periodic_callback(cb)
-        assert len(d.session_callbacks) == 0
-        assert len(events) == 2
-        assert isinstance(events[0], SessionCallbackAdded)
-        assert isinstance(events[1], SessionCallbackRemoved)
-
     def test_add_remove_timeout_callback(self):
         d = document.Document()
 
@@ -614,33 +611,6 @@ class TestDocument(unittest.TestCase):
         assert isinstance(events[0], SessionCallbackAdded)
         assert isinstance(events[1], SessionCallbackRemoved)
 
-    def test_deprecated_add_remove_timeout_callback(self):
-        d = document.Document()
-
-        events = []
-
-        def listener(event):
-            events.append(event)
-
-        d.on_change(listener)
-
-        assert len(d.session_callbacks) == 0
-        assert not events
-
-        def cb(): pass
-
-        d.add_timeout_callback(cb, 1)
-        assert len(d.session_callbacks) == len(events) == 1
-        assert isinstance(events[0], SessionCallbackAdded)
-        assert d.session_callbacks[0] == events[0].callback
-
-        with pytest.warns(BokehDeprecationWarning):
-            d.remove_timeout_callback(cb)
-        assert len(d.session_callbacks) == 0
-        assert len(events) == 2
-        assert isinstance(events[0], SessionCallbackAdded)
-        assert isinstance(events[1], SessionCallbackRemoved)
-
     def test_add_partial_callback(self):
         from functools import partial
         d = document.Document()
@@ -686,33 +656,6 @@ class TestDocument(unittest.TestCase):
         assert isinstance(events[0], SessionCallbackAdded)
         assert isinstance(events[1], SessionCallbackRemoved)
 
-    def test_deprecated_add_remove_next_tick_callback(self):
-        d = document.Document()
-
-        events = []
-
-        def listener(event):
-            events.append(event)
-
-        d.on_change(listener)
-
-        assert len(d.session_callbacks) == 0
-        assert not events
-
-        def cb(): pass
-
-        d.add_next_tick_callback(cb)
-        assert len(d.session_callbacks) == len(events) == 1
-        assert isinstance(events[0], SessionCallbackAdded)
-        assert d.session_callbacks[0] == events[0].callback
-
-        with pytest.warns(BokehDeprecationWarning):
-            d.remove_next_tick_callback(cb)
-        assert len(d.session_callbacks) == 0
-        assert len(events) == 2
-        assert isinstance(events[0], SessionCallbackAdded)
-        assert isinstance(events[1], SessionCallbackRemoved)
-
     def test_periodic_callback_gets_curdoc(self):
         d = document.Document()
         assert curdoc() is not d
@@ -863,39 +806,39 @@ class TestDocument(unittest.TestCase):
                 expected = copy(new_value)
                 if 'units' not in expected:
                     expected['units'] = root1.foo_units
-                self.assertDictEqual(expected, root1.lookup('foo').serializable_value(root1))
+                assert expected == root1.lookup('foo').serializable_value(root1)
             else:
-                self.assertEqual(new_value, root1.foo)
+                assert new_value == root1.foo
         patch_test(57)
-        self.assertEqual('data', root1.foo_units)
+        assert 'data' == root1.foo_units
         patch_test(dict(value=58))
-        self.assertEqual('data', root1.foo_units)
+        assert 'data' == root1.foo_units
         patch_test(dict(value=58, units='screen'))
-        self.assertEqual('screen', root1.foo_units)
+        assert 'screen' == root1.foo_units
         patch_test(dict(value=59, units='screen'))
-        self.assertEqual('screen', root1.foo_units)
+        assert 'screen' == root1.foo_units
         patch_test(dict(value=59, units='data'))
-        self.assertEqual('data', root1.foo_units)
+        assert 'data' == root1.foo_units
         patch_test(dict(value=60, units='data'))
-        self.assertEqual('data', root1.foo_units)
+        assert 'data' == root1.foo_units
         patch_test(dict(value=60, units='data'))
-        self.assertEqual('data', root1.foo_units)
+        assert 'data' == root1.foo_units
         patch_test(61)
-        self.assertEqual('data', root1.foo_units)
+        assert 'data' == root1.foo_units
         root1.foo = "a_string" # so "woot" gets set as a string
         patch_test("woot")
-        self.assertEqual('data', root1.foo_units)
+        assert 'data' == root1.foo_units
         patch_test(dict(field="woot2"))
-        self.assertEqual('data', root1.foo_units)
+        assert 'data' == root1.foo_units
         patch_test(dict(field="woot2", units='screen'))
-        self.assertEqual('screen', root1.foo_units)
+        assert 'screen' == root1.foo_units
         patch_test(dict(field="woot3"))
-        self.assertEqual('screen', root1.foo_units)
+        assert 'screen' == root1.foo_units
         patch_test(dict(value=70))
-        self.assertEqual('screen', root1.foo_units)
+        assert 'screen' == root1.foo_units
         root1.foo = 123 # so 71 gets set as a number
         patch_test(71)
-        self.assertEqual('screen', root1.foo_units)
+        assert 'screen' == root1.foo_units
 
     def test_patch_reference_property(self):
         d = document.Document()
@@ -912,31 +855,31 @@ class TestDocument(unittest.TestCase):
         d.add_root(root2)
         assert len(d.roots) == 2
 
-        assert child1._id in d._all_models
-        assert child2._id not in d._all_models
-        assert child3._id not in d._all_models
+        assert child1.id in d._all_models
+        assert child2.id not in d._all_models
+        assert child3.id not in d._all_models
 
         event1 = ModelChangedEvent(d, root1, 'child', root1.child, child3, child3)
         patch1, buffers = process_document_events([event1])
         d.apply_json_patch_string(patch1)
 
-        assert root1.child._id == child3._id
-        assert root1.child.child._id == child2._id
-        assert child1._id in d._all_models
-        assert child2._id in d._all_models
-        assert child3._id in d._all_models
+        assert root1.child.id == child3.id
+        assert root1.child.child.id == child2.id
+        assert child1.id in d._all_models
+        assert child2.id in d._all_models
+        assert child3.id in d._all_models
 
         # put it back how it was before
         event2 = ModelChangedEvent(d, root1, 'child', root1.child, child1, child1)
         patch2, buffers = process_document_events([event2])
         d.apply_json_patch_string(patch2)
 
-        assert root1.child._id == child1._id
+        assert root1.child.id == child1.id
         assert root1.child.child is None
 
-        assert child1._id in d._all_models
-        assert child2._id not in d._all_models
-        assert child3._id not in d._all_models
+        assert child1.id in d._all_models
+        assert child2.id not in d._all_models
+        assert child3.id not in d._all_models
 
     def test_patch_two_properties_at_once(self):
         d = document.Document()
@@ -982,3 +925,18 @@ class TestDocument(unittest.TestCase):
     # TODO test serialize/deserialize with list-and-dict-valued properties
 
     # TODO test replace_with_json
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
+
+# needed for caplog tests to function
+basicConfig()
