@@ -1,10 +1,35 @@
-import pytest
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
+
+import pytest ; pytest
+
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+
+# External imports
+
+# Bokeh imports
 from mock import patch
-import pandas as pd
 
+# Module under test
 import bokeh.document.events as bde
 
+#-----------------------------------------------------------------------------
+# Setup
+#-----------------------------------------------------------------------------
 
 class FakeEmptyDispatcher(object): pass
 
@@ -26,9 +51,13 @@ class FakeModel(object):
     data = "data"
     def references(self): return dict(ref1=1, ref2=2)
 
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
 # DocumentChangedEvent --------------------------------------------------------
 
-class TesDocumentChangedEvent(object):
+class TestDocumentChangedEvent(object):
 
     def test_init(self):
         e = bde.DocumentChangedEvent("doc")
@@ -270,7 +299,7 @@ class TestColumnsStreamedEvent(object):
         assert e.data == dict(foo=1)
         assert e.rollover == 200
 
-    def test_pandas_data(self):
+    def test_pandas_data(self, pd):
         m = FakeModel()
         df = pd.DataFrame({'x': [1, 2, 3], 'y': [4, 5, 6]})
         e = bde.ColumnsStreamedEvent("doc", m, df, 200, "setter", "invoker")
@@ -475,3 +504,15 @@ class TestSessionCallbackRemoved(object):
         e = bde.SessionCallbackAdded("doc", "setter")
         e2 = bde.SessionCallbackAdded("doc", "setter")
         assert e.combine(e2) == False
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
