@@ -1,7 +1,6 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2017, Anaconda, Inc. All rights reserved.
-#
-# Powered by the Bokeh Development Team.
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
@@ -17,8 +16,6 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import logging
 log = logging.getLogger(__name__)
 
-from bokeh.util.api import general, dev ; general, dev
-
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
@@ -36,6 +33,10 @@ from ..resources import BaseResources
 # Globals and constants
 #-----------------------------------------------------------------------------
 
+__all__ = (
+    'bundle_for_objs_and_resources',
+)
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
@@ -44,7 +45,6 @@ from ..resources import BaseResources
 # Dev API
 #-----------------------------------------------------------------------------
 
-@dev((1,0,0))
 def bundle_for_objs_and_resources(objs, resources):
     ''' Generate rendered CSS and JS resources suitable for the given
     collection of Bokeh objects
