@@ -1,3 +1,9 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Represent granular events that can be used to trigger callbacks.
 
 Bokeh documents and applications are capable of supporting various kinds of
@@ -46,13 +52,58 @@ event object that triggered the callback.
     may trigger at a very high rate.
 
 '''
-from __future__ import absolute_import
+
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import logging
-logger = logging.getLogger(__name__)
+log = logging.getLogger(__name__)
 
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+
+# External imports
+
+# Bokeh imports
 from .util.future import with_metaclass
 
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+__all__ = (
+    'ButtonClick',
+    'DoubleTap',
+    'Event',
+    'LODStart',
+    'LODEnd',
+    'MouseEnter',
+    'MouseLeave',
+    'MouseMove',
+    'MouseWheel',
+    'Pan',
+    'PanEnd',
+    'PanStart',
+    'Pinch',
+    'PinchEnd',
+    'PinchStart',
+    'PlotEvent',
+    'PointEvent',
+    'Press',
+    'Reset',
+    'SelectionGeometry',
+    'Tap',
+)
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
 _CONCRETE_EVENT_CLASSES = dict()
 
 class _MetaEvent(type):
@@ -69,6 +120,10 @@ class _MetaEvent(type):
             _CONCRETE_EVENT_CLASSES[newclass.event_name] = newclass
         return newclass
 
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
 class Event(with_metaclass(_MetaEvent, object)):
     ''' Base class for all Bokeh events.
 
@@ -88,7 +143,7 @@ class Event(with_metaclass(_MetaEvent, object)):
         '''
         self._model_id = None
         if model is not None:
-            self._model_id = model._id
+            self._model_id = model.id
 
     @classmethod
     def decode_json(cls, dct):
@@ -143,7 +198,7 @@ class ButtonClick(Event):
         super(ButtonClick, self).__init__(model=model)
 
 class PlotEvent(Event):
-    ''' PlotEvent is the base class for all events applicable to Plot models.
+    ''' The base class for all events applicable to Plot models.
 
     '''
 
@@ -196,7 +251,7 @@ class SelectionGeometry(PlotEvent):
         super(SelectionGeometry, self).__init__(model=model)
 
 class Reset(PlotEvent):
-    ''' Announce a button click event on a plot ResetTool.
+    ''' Announce a button click event on a plot ``ResetTool``.
 
     '''
     event_name = "reset"
@@ -227,8 +282,6 @@ class PointEvent(PlotEvent):
         self.y = y
         super(PointEvent, self).__init__(model=model)
 
-# --- Point Events ------------------------------------------------------------
-
 class Tap(PointEvent):
     ''' Announce a tap or click event on a Bokeh plot.
 
@@ -326,8 +379,8 @@ class MouseWheel(PointEvent):
 
     .. note::
         By default, Bokeh plots do not prevent default scroll events unless a
-        WheelZoomTool or WheelPanTool is active. This may change in future
-        releases.
+        ``WheelZoomTool`` or ``WheelPanTool`` is active. This may change in
+        future releases.
 
     '''
     event_name = 'wheel'
@@ -430,3 +483,11 @@ class PinchStart(PointEvent):
 
     '''
     event_name = 'pinchstart'
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
