@@ -1,7 +1,6 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2017, Anaconda, Inc. All rights reserved.
-#
-# Powered by the Bokeh Development Team.
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
@@ -17,8 +16,6 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import logging
 log = logging.getLogger(__name__)
 
-from bokeh.util.api import general, dev ; general, dev
-
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
@@ -31,6 +28,7 @@ from warnings import warn
 # External imports
 
 # Bokeh imports
+from ..settings import settings
 from ..util.string import decode_utf8
 from .state import curstate
 from .util import default_filename
@@ -41,12 +39,15 @@ from .util import default_filename
 
 DEFAULT_TITLE = "Bokeh Plot"
 
+__all__ = (
+    'save',
+)
+
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
-@general((1,0,0))
-def save(obj, filename=None, resources=None, title=None, state=None, **kwargs):
+def save(obj, filename=None, resources=None, title=None, template=None, state=None, **kwargs):
     ''' Save an HTML file with the data for the current document.
 
     Will fall back to the default output state (or an explicitly provided
@@ -82,7 +83,7 @@ def save(obj, filename=None, resources=None, title=None, state=None, **kwargs):
         state = curstate()
 
     filename, resources, title = _get_save_args(state, filename, resources, title)
-    _save_helper(obj, filename, resources, title)
+    _save_helper(obj, filename, resources, title, template)
     return abspath(filename)
 
 #-----------------------------------------------------------------------------
@@ -109,7 +110,7 @@ def _get_save_filename(state, filename):
     if filename is not None:
         return filename, False
 
-    if state.file:
+    if state.file and not settings.ignore_filename():
         return state.file['filename'], False
 
     return default_filename("html"), True
@@ -139,12 +140,12 @@ def _get_save_title(state, title, suppress_warning):
 
     return DEFAULT_TITLE
 
-def _save_helper(obj, filename, resources, title):
+def _save_helper(obj, filename, resources, title, template):
     '''
 
     '''
     from ..embed import file_html
-    html = file_html(obj, resources, title=title)
+    html = file_html(obj, resources, title=title, template=template)
 
     with io.open(filename, mode="w", encoding="utf-8") as f:
         f.write(decode_utf8(html))
