@@ -1,4 +1,9 @@
-# -*- coding: utf-8 -*-
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Display a variety of visual shapes whose attributes can be associated
 with data columns from ``ColumnDataSources``.
 
@@ -10,11 +15,13 @@ The full list of glyphs built into Bokeh is given below:
 * :class:`~bokeh.models.glyphs.Bezier`
 * :class:`~bokeh.models.glyphs.Ellipse`
 * :class:`~bokeh.models.glyphs.HBar`
+* :class:`~bokeh.models.glyphs.HexTile`
 * :class:`~bokeh.models.glyphs.Image`
 * :class:`~bokeh.models.glyphs.ImageRGBA`
 * :class:`~bokeh.models.glyphs.ImageURL`
 * :class:`~bokeh.models.glyphs.Line`
 * :class:`~bokeh.models.glyphs.MultiLine`
+* :class:`~bokeh.models.glyphs.MultiPolygons`
 * :class:`~bokeh.models.glyphs.Oval`
 * :class:`~bokeh.models.glyphs.Patch`
 * :class:`~bokeh.models.glyphs.Patches`
@@ -35,17 +42,71 @@ All these glyphs share a minimal common interface through their base class
     :members:
 
 '''
-from __future__ import absolute_import
 
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
+
+import logging
+log = logging.getLogger(__name__)
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+
+# External imports
+
+# Bokeh imports
 from ..core.enums import Anchor, Direction, StepMode
 from ..core.has_props import abstract
-from ..core.properties import (AngleSpec, Bool, DistanceSpec, Enum, Float,
-                               Include, Instance, Int, NumberSpec, StringSpec)
+from ..core.properties import (AngleSpec, Bool, DistanceSpec, Enum, Float, String,
+                               Include, Instance, Int, NumberSpec, Override, StringSpec)
 from ..core.property_mixins import FillProps, LineProps, ScalarFillProps, ScalarLineProps, TextProps
 from ..model import Model
 
 from .mappers import ColorMapper, LinearColorMapper
 
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+__all__ = (
+    'AnnularWedge',
+    'Annulus',
+    'Arc',
+    'Bezier',
+    'Ellipse',
+    'Glyph',
+    'HBar',
+    'HexTile',
+    'Image',
+    'ImageRGBA',
+    'ImageURL',
+    'Line',
+    'MultiLine',
+    'MultiPolygons',
+    'Oval',
+    'Patch',
+    'Patches',
+    'Quad',
+    'Quadratic',
+    'Ray',
+    'Rect',
+    'Segment',
+    'Step',
+    'Text',
+    'VBar',
+    'Wedge',
+    'XYGlyph',
+)
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
 @abstract
 class Glyph(Model):
     ''' Base class for all glyph models.
@@ -180,11 +241,11 @@ class Arc(XYGlyph):
     """)
 
 class Bezier(Glyph):
-    u''' Render Bézier curves.
+    ''' Render Bezier curves.
 
-    For more information consult the `Wikipedia article for Bézier curve`_.
+    For more information consult the `Wikipedia article for Bezier curve`_.
 
-    .. _Wikipedia article for Bézier curve: http://en.wikipedia.org/wiki/Bézier_curve
+    .. _Wikipedia article for Bezier curve: http://en.wikipedia.org/wiki/Bezier_curve
 
     '''
 
@@ -226,12 +287,12 @@ class Bezier(Glyph):
     The y-coordinates of second control points.
     """)
 
-    line_props = Include(LineProps, use_prefix=False, help=u"""
-    The %s values for the Bézier curves.
+    line_props = Include(LineProps, use_prefix=False, help="""
+    The %s values for the Bezier curves.
     """)
 
 class Ellipse(XYGlyph):
-    u''' Render ellipses.
+    ''' Render ellipses.
 
     '''
 
@@ -305,6 +366,59 @@ class HBar(Glyph):
     The %s values for the horizontal bars.
     """)
 
+class HexTile(Glyph):
+    ''' Render horizontal tiles on a regular hexagonal grid.
+
+    '''
+
+    __example__ = "examples/reference/models/HexTile.py"
+
+    # a canonical order for positional args that can be used for any
+    # functions derived from this class
+    _args = ('q', 'r')
+
+    size = Float(1.0, help="""
+    The radius (in data space units) of the hex tiling.
+
+    The radius is always measured along the cartesian y-axis for "pointy_top"
+    orientation, and along the cartesian x-axis for "flat_top" orientation. If
+    the aspect ratio of the underlying cartesian system is not 1-1, then the
+    tiles may be "squished" in one direction. To ensure that the tiles are
+    always regular hexagons, consider setting the ``match_aspect`` property of
+    the plot to True.
+    """)
+
+    aspect_scale = Float(1.0, help="""
+
+    """)
+
+    r = NumberSpec(help="""
+    The "row" axial coordinates of the tile centers.
+    """)
+
+    q = NumberSpec(help="""
+    The "column" axial coordinates of the tile centers.
+    """)
+
+    scale = NumberSpec(1.0, help="""
+    A scale factor for individual tiles.
+    """)
+
+    orientation = String("pointytop", help="""
+
+    """)
+
+    line_props = Include(LineProps, use_prefix=False, help="""
+    The %s values for the horizontal bars.
+    """)
+
+    line_color = Override(default=None)
+
+    fill_props = Include(FillProps, use_prefix=False, help="""
+    The %s values for the horizontal bars.
+    """)
+
+
 class Image(XYGlyph):
     ''' Render images given as scalar data together with a color mapper.
 
@@ -333,7 +447,7 @@ class Image(XYGlyph):
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
-    _args = ('image', 'x', 'y', 'dw', 'dh', 'dilate')
+    _args = ('image', 'x', 'y', 'dw', 'dh', 'global_alpha', 'dilate')
 
     # a hook to specify any additional kwargs handled by an initializer
     _extra_kws = {
@@ -371,6 +485,11 @@ class Image(XYGlyph):
         That number is fixed by the image itself.
     """)
 
+    global_alpha = Float(1.0, help="""
+    An overall opacity that each image is rendered with (in addition
+    to any alpha values applied explicitly in a color mapper).
+    """)
+
     dilate = Bool(False, help="""
     Whether to always round fractional pixel locations in such a way
     as to make the images bigger.
@@ -397,7 +516,7 @@ class ImageRGBA(XYGlyph):
 
     # a canonical order for positional args that can be used for any
     # functions derived from this class
-    _args = ('image', 'x', 'y', 'dw', 'dh', 'dilate')
+    _args = ('image', 'x', 'y', 'dw', 'dh', 'global_alpha', 'dilate')
 
     image = NumberSpec(help="""
     The arrays of RGBA data for the images.
@@ -427,6 +546,11 @@ class ImageRGBA(XYGlyph):
         That number is fixed by the image itself.
     """)
 
+    global_alpha = Float(1.0, help="""
+    An overall opacity that each image is rendered with (in addition
+    to any inherent alpha values in the image itself).
+    """)
+
     dilate = Bool(False, help="""
     Whether to always round fractional pixel locations in such a way
     as to make the images bigger.
@@ -449,8 +573,7 @@ class ImageURL(XYGlyph):
     # functions derived from this class
     _args = ('url', 'x', 'y', 'w', 'h', 'angle', 'global_alpha', 'dilate')
 
-    # TODO (bev) Why is this a NumberSpec??
-    url = NumberSpec(accept_datetime=False, help="""
+    url = StringSpec(default=None, help="""
     The URLs to retrieve images from.
 
     .. note::
@@ -467,7 +590,7 @@ class ImageURL(XYGlyph):
     """)
 
     w = DistanceSpec(default=None, help="""
-    The height of the plot region that the image will occupy in data space.
+    The width of the plot region that the image will occupy in data space.
 
     The default value is ``None``, in which case the image will be displayed
     at its actual image size (regardless of the units specified here).
@@ -563,10 +686,50 @@ class MultiLine(Glyph):
     The %s values for the lines.
     """)
 
-class Oval(XYGlyph):
-    u''' Render ovals.
+class MultiPolygons(Glyph):
+    ''' Render several MultiPolygon.
 
-    This glyph renders ovals using Bézier curves, which are similar,
+    Modeled on geoJSON - the data for the ``MultiPolygons`` glyph is
+    different in that the vector of values is not a vector of scalars.
+    Rather, it is a "list of lists of lists of lists".
+    '''
+
+    __example__ = "examples/reference/models/MultiPolygons.py"
+
+    # a canonical order for positional args that can be used for any
+    # functions derived from this class
+    _args = ('xs', 'ys')
+
+    xs = NumberSpec(help="""
+    The x-coordinates for all the patches, given as a nested list.
+
+    .. note::
+        Each item in ``MultiPolygons`` represents one MultiPolygon and each
+        MultiPolygon is comprised of ``n`` Polygons. Each Polygon is made of
+        one exterior ring optionally followed by ``m`` interior rings (holes).
+    """)
+
+    ys = NumberSpec(help="""
+    The y-coordinates for all the patches, given as a "list of lists".
+
+    .. note::
+        Each item in ``MultiPolygons`` represents one MultiPolygon and each
+        MultiPolygon is comprised of ``n`` Polygons. Each Polygon is made of
+        one exterior ring optionally followed by ``m`` interior rings (holes).
+    """)
+
+    line_props = Include(LineProps, use_prefix=False, help="""
+    The %s values for the multipolygons.
+    """)
+
+    fill_props = Include(FillProps, use_prefix=False, help="""
+    The %s values for the multipolygons.
+    """)
+
+class Oval(XYGlyph):
+    ''' Render ovals.
+
+    This glyph renders ovals using Bezier curves, which are similar,
     but not identical to ellipses. In particular, widths equal to heights
     will not render circles. Use the ``Ellipse`` glyph for that.
 
@@ -1032,10 +1195,23 @@ class Wedge(XYGlyph):
     The %s values for the wedges.
     """)
 
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
+
 # XXX: allow `from bokeh.models.glyphs import *`
-from .markers import (Asterisk, Circle, CircleCross, CircleX, Cross, Diamond, DiamondCross,
-                      InvertedTriangle, Marker, Square, SquareCross, SquareX, Triangle, X)
+from .markers import (Asterisk, Circle, CircleCross, CircleX, Cross, Dash,
+                      Diamond, DiamondCross, Hex, InvertedTriangle, Marker,
+                      Square, SquareCross, SquareX, Triangle, X)
 
 # Fool pyflakes
-(Asterisk, Circle, CircleCross, CircleX, Cross, Diamond, DiamondCross,
-InvertedTriangle, Marker, Square, SquareCross, SquareX, Triangle, X)
+(Asterisk, Circle, CircleCross, CircleX, Cross, Dash, Diamond, DiamondCross,
+Hex, InvertedTriangle, Marker, Square, SquareCross, SquareX, Triangle, X)
