@@ -1,12 +1,31 @@
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
 ''' Models (mostly base classes) for the various kinds of renderer
 types that Bokeh supports.
 
 '''
-from __future__ import absolute_import
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
 
+from difflib import get_close_matches
 import logging
-logger = logging.getLogger(__name__)
+log = logging.getLogger(__name__)
 
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
+
+# External imports
+
+# Bokeh imports
 from ..core.enums import RenderLevel
 from ..core.has_props import abstract
 from ..core.properties import Auto, Bool, Either, Enum, Float, Instance, Override, String
@@ -20,6 +39,27 @@ from .graphs import LayoutProvider, GraphHitTestPolicy, NodesOnly
 from .sources import ColumnDataSource, DataSource, RemoteSource, CDSView
 from .tiles import TileSource, WMTSTileSource
 
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+__all__ = (
+    'DataRenderer',
+    'GlyphRenderer',
+    'GraphRenderer',
+    'GuideRenderer',
+    'Renderer',
+    'TileRenderer',
+)
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
 @abstract
 class Renderer(Model):
     '''An abstract base class for renderer types.
@@ -53,6 +93,10 @@ class TileRenderer(DataRenderer):
     tile opacity 0.0 - 1.0
     """)
 
+    smoothing = Bool(default=True, help="""
+    Enable image smoothing for the rendered tiles.
+    """)
+
     x_range_name = String('default', help="""
     A particular (named) x-range to use for computing screen
     locations when rendering glyphs on the plot. If unset, use the
@@ -93,15 +137,19 @@ class GlyphRenderer(DataRenderer):
         if not self.glyph: return
         if not self.data_source: return
         if isinstance(self.data_source, RemoteSource): return
-        missing = set()
+        missing_values = set()
         specs = self.glyph.dataspecs()
         for name, item in self.glyph.properties_with_values(include_defaults=False).items():
             if name not in specs: continue
             if not isinstance(item, dict): continue
             if not isinstance(self.data_source, ColumnDataSource): continue
             if 'field' in item and item['field'] not in self.data_source.column_names:
-                missing.add(item['field'])
-        if missing:
+                missing_values.add((item['field'], name))
+        if missing_values:
+            suggestions = ['" (closest match: "%s")' % s[0] if s else '"' for s in [
+                get_close_matches(term[0], self.data_source.column_names, n=1) for term in missing_values]]
+            missing_values = [("".join([m[0], s]), m[1]) for m, s in zip(missing_values, suggestions)]
+            missing = ['key "%s" value "%s' % (k, v) for v, k in missing_values]
             return "%s [renderer: %s]" % (", ".join(sorted(missing)), self)
 
     def __init__(self, **kw):
@@ -128,7 +176,7 @@ class GlyphRenderer(DataRenderer):
     y_range_name = String('default', help="""
     A particular (named) y-range to use for computing screen
     locations when rendering glyphs on the plot. If unset, use the
-    default -range.
+    default y-range.
     """)
 
     glyph = Instance(Glyph, help="""
@@ -154,7 +202,7 @@ class GlyphRenderer(DataRenderer):
 
     hover_glyph = Instance(Glyph, help="""
     An optional glyph used for inspected points, e.g., those that are
-    being hovered over by a HoverTool.
+    being hovered over by a ``HoverTool``.
     """)
 
     muted_glyph = Instance(Glyph, help="""
@@ -199,31 +247,31 @@ class GraphRenderer(DataRenderer):
     y_range_name = String('default', help="""
     A particular (named) y-range to use for computing screen
     locations when rendering graphs on the plot. If unset, use the
-    default -range.
+    default y-range.
     """)
 
     layout_provider = Instance(LayoutProvider, help="""
-    An instance of a LayoutProvider that supplies the layout of the network
+    An instance of a ``LayoutProvider`` that supplies the layout of the network
     graph in cartesian space.
     """)
 
     node_renderer = Instance(GlyphRenderer, default=_DEFAULT_NODE_RENDERER, help="""
-    Instance of a GlyphRenderer containing an XYGlyph that will be rendered
+    Instance of a ``GlyphRenderer`` containing an ``XYGlyph`` that will be rendered
     as the graph nodes.
     """)
 
     edge_renderer = Instance(GlyphRenderer, default=_DEFAULT_EDGE_RENDERER, help="""
-    Instance of a GlyphRenderer containing an MultiLine Glyph that will be
+    Instance of a ``GlyphRenderer`` containing an ``MultiLine`` Glyph that will be
     rendered as the graph edges.
     """)
 
     selection_policy = Instance(GraphHitTestPolicy, default=lambda: NodesOnly(), help="""
-    An instance of a GraphHitTestPolicy that provides the logic for selection
+    An instance of a ``GraphHitTestPolicy`` that provides the logic for selection
     of graph components.
     """)
 
     inspection_policy = Instance(GraphHitTestPolicy, default=lambda: NodesOnly(), help="""
-    An instance of a GraphHitTestPolicy that provides the logic for inspection
+    An instance of a ``GraphHitTestPolicy`` that provides the logic for inspection
     of graph components.
     """)
 
@@ -248,3 +296,11 @@ class GuideRenderer(Renderer):
                 self.plot.renderers.append(self)
 
     level = Override(default="overlay")
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
