@@ -1,21 +1,66 @@
-""" Various kinds of slider widgets.
+#-----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# All rights reserved.
+#
+# The full license is in the file LICENSE.txt, distributed with this software.
+#-----------------------------------------------------------------------------
+''' Various kinds of slider widgets.
 
-"""
-from __future__ import absolute_import
+'''
 
+#-----------------------------------------------------------------------------
+# Boilerplate
+#-----------------------------------------------------------------------------
+from __future__ import absolute_import, division, print_function, unicode_literals
+
+import logging
+log = logging.getLogger(__name__)
+
+#-----------------------------------------------------------------------------
+# Imports
+#-----------------------------------------------------------------------------
+
+# Standard library imports
 from datetime import datetime
 import numbers
 
+# External imports
+
+# Bokeh imports
 from ...core.has_props import abstract
 from ...core.properties import Bool, Int, Float, String, Date, Enum, Tuple, Instance, Color, Override
 from ...core.enums import SliderCallbackPolicy
+from ...core.validation import error
+from ...core.validation.errors import EQUAL_SLIDER_START_END
 from ..callbacks import Callback
 from .widget import Widget
 
+#-----------------------------------------------------------------------------
+# Globals and constants
+#-----------------------------------------------------------------------------
+
+__all__ = (
+    'AbstractSlider',
+    'Slider',
+    'RangeSlider',
+    'DateSlider',
+    'DateRangeSlider',
+)
+
+#-----------------------------------------------------------------------------
+# Dev API
+#-----------------------------------------------------------------------------
+
 @abstract
 class AbstractSlider(Widget):
     """ """
 
+    def __init__(self, **kwargs):
+        if 'start' in kwargs and 'end' in kwargs:
+            if kwargs['start'] == kwargs['end']:
+                raise ValueError("Slider 'start' and 'end' cannot be equal.")
+        super(Widget, self).__init__(**kwargs)
+
     title = String(default="", help="""
     Slider's label.
     """)
@@ -42,7 +87,7 @@ class AbstractSlider(Widget):
     """)
 
     callback_throttle = Float(default=200, help="""
-    Number of millseconds to pause between callback calls as the slider is moved.
+    Number of milliseconds to pause between callback calls as the slider is moved.
     """)
 
     callback_policy = Enum(SliderCallbackPolicy, default="throttle", help="""
@@ -53,11 +98,24 @@ class AbstractSlider(Widget):
     * "mouseup": the callback will be executed only once when the slider is released.
 
     The "mouseup" policy is intended for scenarios in which the callback is expensive in time.
+
+    .. warning::
+        Callback policy currently apply to JS callbacks
     """)
 
     bar_color = Color(default="#e6e6e6", help="""
     """)
 
+    @error(EQUAL_SLIDER_START_END)
+    def _check_missing_dimension(self):
+        if hasattr(self, 'start') and hasattr(self, 'end'):
+            if self.start == self.end:
+                return '{!s} with title {!s}'.format(self, self.title)
+
+#-----------------------------------------------------------------------------
+# General API
+#-----------------------------------------------------------------------------
+
 class Slider(AbstractSlider):
     """ Slider-based number selection widget. """
 
@@ -119,7 +177,7 @@ class DateSlider(AbstractSlider):
     The step between consecutive values.
     """)
 
-    format = Override(default="%d %b %G")
+    format = Override(default="%d %b %Y")
 
 class DateRangeSlider(AbstractSlider):
     """ Slider-based date range selection widget. """
@@ -159,4 +217,12 @@ class DateRangeSlider(AbstractSlider):
     The step between consecutive values.
     """)
 
-    format = Override(default="%d %b %G")
+    format = Override(default="%d %b %Y")
+
+#-----------------------------------------------------------------------------
+# Private API
+#-----------------------------------------------------------------------------
+
+#-----------------------------------------------------------------------------
+# Code
+#-----------------------------------------------------------------------------
